@@ -281,12 +281,7 @@ static int raw_init(struct sock *sk)
 static int raw_release(struct socket *sock)
 {
 	struct sock *sk = sock->sk;
-	struct raw_sock *ro;
-
-	if (!sk)
-		return 0;
-
-	ro = raw_sk(sk);
+	struct raw_sock *ro = raw_sk(sk);
 
 	unregister_netdevice_notifier(&ro->notifier);
 
