@@ -24,6 +24,7 @@
 #include <linux/kernel.h>
 #include <linux/jiffies.h>
 #include <linux/timer.h>
+#include <linux/slab.h>
 #include <linux/netdevice.h>
 #include <linux/skbuff.h>
 #include <asm/uaccess.h>
@@ -391,12 +392,8 @@ void __exit x25_link_free(void)
 	write_lock_bh(&x25_neigh_list_lock);
 
 	list_for_each_safe(entry, tmp, &x25_neigh_list) {
-		struct net_device *dev;
-
 		nb = list_entry(entry, struct x25_neigh, node);
-		dev = nb->dev;
 		__x25_remove_neigh(nb);
-		dev_put(dev);
 	}
 	write_unlock_bh(&x25_neigh_list_lock);
 }
